@@ -3,13 +3,14 @@ import os
 import socket
 from distutils.version import StrictVersion
 import uuid
+from urllib.parse import quote
+import json
 
 import pandas as pd
 import sys
-from typing import Dict
-
 import pyjava.utils as utils
 import requests
+
 from pyjava.serializers import ArrowStreamSerializer
 from pyjava.serializers import read_int
 from pyjava.utils import utf8_deserializer
@@ -45,13 +46,12 @@ class LogClient(object):
             else:
                 logging.info(msg)
             return
-        import json
         resp = json.dumps(
             {"sendLog": {
                 "token": self.log_token,
                 "logLine": "[owner] [{}] [groupId] [{}] __MMMMMM__ {}".format(self.log_user, self.log_group_id, msg)
             }}, ensure_ascii=False)
-        requests.post(self.url, data=resp, headers={'content-type': 'application/x-www-form-urlencoded;charset=UTF-8'})
+        requests.post(self.url, data=resp, headers={'Content-Type': 'application/json;charset=UTF-8'})
 
     def close(self):
         if hasattr(self, "conn"):
@@ -396,6 +396,11 @@ add comment like: `#%dataMode=data` if you are in notebook.
         items = [row for row in self.collect()]
         return pd.DataFrame(data=items)
 
+    def to_dataset(self, num_tables_per_block: int = 1):
+        from pyjava.data.datasource import KoloRawDatasource
+        source = KoloRawDatasource(self.servers, num_tables_per_block)
+        return source.to_dataset()
+
     @staticmethod
     def fetch_once_as_rows(data_server):
         for df in RayContext.fetch_data_from_single_data_server(data_server):
@@ -404,6 +409,11 @@ add comment like: `#%dataMode=data` if you are in notebook.
 
     @staticmethod
     def fetch_data_from_single_data_server(data_server):
+        for table in RayContext.fetch_data_from_single_data_server_as_arrow(data_server):
+            yield table.to_pandas()
+
+    @staticmethod
+    def fetch_data_from_single_data_server_as_arrow(data_server):
         out_ser = ArrowStreamSerializer()
         import pyarrow as pa
         with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
@@ -412,4 +422,4 @@ add comment like: `#%dataMode=data` if you are in notebook.
             infile = os.fdopen(os.dup(sock.fileno()), "rb", buffer_size)
             result = out_ser.load_stream(infile)
             for items in result:
-                yield pa.Table.from_batches([items]).to_pandas()
+                yield pa.Table.from_batches([items])
